;*********************************************************************************************
; Software License Agreement                                                                 *
; The software supplied herewith by Microchip Technology Incorporated                        *
; (the "Company") is intended and supplied to you, the Company's                             *
; customer, for use solely and exclusively on Microchip products.                            *
;                                                                                            *
; The software is owned by the Company and/or its supplier, and is                           *
; protected under applicable copyright laws. All rights are reserved.                        *
; Any use in violation of the foregoing restrictions may subject the                         *
; user to criminal sanctions under applicable laws, as well as to                            *
; civil liability for the breach of the terms and conditions of this                         *
; license.                                                                                   *
;                                                                                            *
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,                          *
; WHETHER EXPRESS, IMPLIED OR STATU-TORY, INCLUDING, BUT NOT LIMITED                         *
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A                                *
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,                          *
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR                                 *
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.                                          *
;                                                                                            *
;*********************************************************************************************
; Filename:  Demo.asm																		 *
; This demo application uses PWM generation software to drive a RGB							 *
; LED with a 12F615 device.  The PWM code is in SoftwarePWM.asm.							 *
; This file initializes the PWM routines and modulates 3 PWM duty							 *
; cycles with a sinusoidal profile.  The sinusoidal profile causes							 *	
; the RGB LED to sweep through a range of colors.  A potentiometer 							 *
; voltage is sampled once per PWM period and the conversion result							 *
; is used to set the color sweep speed.														 *
;*********************************************************************************************
		list      p=12F615        ; list directive to define processor
		#include <p12F615.inc>    ; processor specific variable definitions
		#include <SoftwarePWM.h>
		errorlevel  -302          ; suppress message 302 from list file

	__CONFIG _CP_OFF & _WDT_OFF & _BOR_ON & _MCLRE_ON & _PWRTE_ON & _INTRC_OSC_NOCLKOUT


#define 	TABLESIZE   .128

	UDATA
Temp	res 1
PhaseL	res 1		; PhaseL and PhaseH hold the sinusoidal 
PhaseH	res 1		; pointer location


	extern	Dutycycle0, Dutycycle1, Dutycycle2, PWMFlags, ISRPWM
	extern  InitPWM, IntCount



;******************************************************************************
;
;Code starts to run from here	
;This will placed by the linker inside the memory location bet'n 0 and 4

STARTUP	CODE 0x0000	

Reset				;Reset Vector
  	goto	Main

	nop
	nop
	nop
	
Interrupt 			;Interrupt Vector
   	goto 	ISRPWM
 
 
 
;******************************************************************************
;
; Main code section starts to run from here
PROG CODE 					;For the rest of the code 
	  	
Main

	call	InitPWM

	
	BANK1					;Select the RAM memory bank 1
	
	;Configure the input mode of an I/O pin to analog.
	bsf		ANSEL,0			;Pot input
	bsf		ANSEL,1			;For bus voltage feedback

	; Setup ANSEL bits for ADC clock period
	bsf		ANSEL,4			;Clock derived from the internal oscillator 
	bsf		ANSEL,5
	bcf		ANSEL,6	

	BANK0
	;A/D Module setup
	movlw	b'00000001'
			 ;0-------		 ADFM->0: Left justified
			 ;-0------		 VCFG->0: VDD as Voltage Reference
			 ;--00----		 Unimplemented bits
			 ;----00--		 CHS1:CHS0->00: AN0 as Analog channel select
			 ;------0-		 GO/!DONE->0: A/D conversion not in progress or done
			 ;-------1		 ADON->1: A/D conversion enabled
	movwf	ADCON0 

	clrf	PhaseH			; Initialize the sinusoidal pointer to 0
	clrf	PhaseL
	
; The main program loop starts here.  The process of PWM signal 
; generation is driven by Timer0 interrupts.  A software flag
; called 'BeginBWM' is set in the Timer0 ISR at the start of 
; a new PWM period.  When the flag is detected in the main loop,
; the potentiometer is sampled, the sinusoidal pointer is incremented,
; and new duty cycle values are determined for the three PWM signals.
MainLoop
	btfss	BeginPWM
	goto	MainLoop
	bcf		BeginPWM
	
ADCConversion
	bsf		ADCON0,GO		;Start A2D conversion
	btfsc	ADCON0,GO		;Wait until the conversion is completed	
	goto	$-1

; The upper 8 bits of the ADC result is added to the 16-bit sine pointer,
; PhaseH:PhaseL.  An offset of 25 is also added to make sure the sweep
; frequency never goes to 0. 
	rrf		ADRESH,W		; use only the upper 7 bits of ADC result
	andlw	0x7f			; mask off upper bit to 0 after rotate
	movwf	Temp
	movlw	.25				;ADRESH + 25
	addwf	Temp,W
	addwf	PhaseL,F		;PhaseL+Temp
	btfsc	STATUS,C		;Chk if the Carry is generated because of the 
							;addition of PhaseL and Temp in the previous 
							;instruction
	incf	PhaseH,F	
	
; To get the three duty cycle values, The upper 7 bits of PhaseH are
; used as a lookup index to the sine table.  Offsets of 0x55 (120
; degrees) and 0xAA (240 degrees are used for the 2nd and 3rd duty
; cycles.
ForPhase1
	rrf		PhaseH,W		 
	andlw	0x7f						
	call	SineTable
	movwf	Dutycycle0
	
ForPhase2
	rrf		PhaseH,W		 
	addlw	0x55			;Phase2 = Phase1 + 0x55 for the first phase shift of 120 degree
	andlw	0x7f
	call 	SineTable
	movwf	Dutycycle1

ForPhase3
	rrf		PhaseH,W		 
	addlw	0xAA			;Phase3 = Phase1 + 0xAA for the second phase shift of 240 degree
	andlw	0x7f
	call 	SineTable
	movwf	Dutycycle2

	goto	MainLoop


; This is a 128 entry table containing 1 cycle of a sine wave.
; A 7-bit table index value is required.
SineTable
	andlw	TABLESIZE-1			;Mask off invalid entries
	movwf	Temp
	movlw	high (TableStart)	;get upper address bits for start of table
	movwf	PCLATH
	movlw	low (TableStart)	;load lower address bits of table
	addwf	Temp,w				;add offset.
	btfsc	STATUS,C			;did it overflow?
	incf	PCLATH,f			;yes:increment PCLATH
	movwf	PCL					;perform a complete jump

TableStart
	retlw	.16
	retlw	.17
	retlw	.17
	retlw	.18
	retlw	.19
	retlw	.20
	retlw	.20
	retlw	.21
	retlw	.22
	retlw	.22
	retlw	.23
	retlw	.24
	retlw	.24
	retlw	.25
	retlw	.26
	retlw	.26
	retlw	.27
	retlw	.27
	retlw	.28
	retlw	.28
	retlw	.28
	retlw	.29
	retlw	.29
	retlw	.30
	retlw	.30
	retlw	.30
	retlw	.30
	retlw	.31
	retlw	.31
	retlw	.31
	retlw	.31
	retlw	.31
	retlw	.31
	retlw	.31
	retlw	.31
	retlw	.31
	retlw	.31
	retlw	.31
	retlw	.30
	retlw	.30
	retlw	.30
	retlw	.30
	retlw	.29
	retlw	.29
	retlw	.28
	retlw	.28
	retlw	.28
	retlw	.27
	retlw	.27
	retlw	.26
	retlw	.26
	retlw	.25
	retlw	.24
	retlw	.24
	retlw	.23
	retlw	.22
	retlw	.22
	retlw	.21
	retlw	.20
	retlw	.20
	retlw	.19
	retlw	.18
	retlw	.17
	retlw	.17
	retlw	.16
	retlw	.15
	retlw	.15
	retlw	.14
	retlw	.13
	retlw	.12
	retlw	.12
	retlw	.11
	retlw	.10
	retlw	.10
	retlw	.09
	retlw	.08
	retlw	.08
	retlw	.07
	retlw	.06
	retlw	.06
	retlw	.05
	retlw	.05
	retlw	.04
	retlw	.04
	retlw	.04
	retlw	.03
	retlw	.03
	retlw	.02
	retlw	.02
	retlw	.02
	retlw	.02
	retlw	.01
	retlw	.01
	retlw	.01
	retlw	.01
	retlw	.01
	retlw	.01
	retlw	.01
	retlw	.01
	retlw	.01
	retlw	.01
	retlw	.01
	retlw	.02
	retlw	.02
	retlw	.02
	retlw	.02
	retlw	.03
	retlw	.03
	retlw	.04
	retlw	.04
	retlw	.04
	retlw	.05
	retlw	.05
	retlw	.06
	retlw	.06
	retlw	.07
	retlw	.08
	retlw	.08
	retlw	.09
	retlw	.10
	retlw	.10
	retlw	.11
	retlw	.12
	retlw	.12
	retlw	.13
	retlw	.14
	retlw	.15
	retlw	.15
	
	END
